@@ -1,12 +1,13 @@
-//! Chip flashing logic.
+//! Chip flashing routine
 use std::thread::sleep;
 use std::time::Duration;
 
 use anyhow::Result;
-use scroll::{Pread, LE};
+use scroll::{Pread, Pwrite, LE};
 
 use crate::{
     constants::{CFG_MASK_ALL, CFG_MASK_RDPR_USER_DATA_WPR, SECTOR_SIZE},
+    device::{parse_number, ChipDB},
     transport::UsbTransport,
     Chip, Command, Transport,
 };
@@ -28,14 +29,17 @@ impl Flashing<UsbTransport> {
         let identify = Command::identify(0, 0);
         let resp = transport.transfer(identify)?;
         anyhow::ensure!(resp.is_ok(), "idenfity chip failed");
-        let chip = Chip::guess(resp.payload()[0], resp.payload()[1])?;
+
+        let chip_db = ChipDB::load()?;
+
+        let chip = chip_db.find_chip(resp.payload()[0], resp.payload()[1])?;
         log::debug!("found chip: {}", chip);
 
         let read_conf = Command::read_config(CFG_MASK_ALL);
         let resp = transport.transfer(read_conf)?;
         anyhow::ensure!(resp.is_ok(), "read_config failed");
 
-        log::debug!("read_config: {}", hex::encode(&resp.payload()));
+        log::debug!("read_config: {}", hex::encode(&resp.payload()[2..]));
         let code_flash_protected = chip.support_code_flash_protect() && resp.payload()[2] != 0xa5;
         let mut btver = [0u8; 4];
         btver.copy_from_slice(&resp.payload()[14..18]);
@@ -65,13 +69,29 @@ impl Flashing<UsbTransport> {
 }
 
 impl<T: Transport> Flashing<T> {
+
+    pub fn check_chip_name(&self, name: &str) -> Result<()> {
+        if !self.chip.name.starts_with(name)  {
+            anyhow::bail!("chip name mismatch: {}", self.chip.name);
+        }
+        Ok(())
+    }
+
     pub fn dump_info(&mut self) -> Result<()> {
-        log::info!(
-            "Chip: {} (Code Flash: {}KiB, Data EEPROM: {}KiB)",
-            self.chip,
-            self.chip.max_code_flash_size / 1024,
-            self.chip.max_data_flash_size / 1024
-        );
+        if self.chip.eeprom_size > 0 {
+            log::info!(
+                "Chip: {} (Code Flash: {}KiB, Data EEPROM: {}KiB)",
+                self.chip,
+                self.chip.flash_size / 1024,
+                self.chip.eeprom_size / 1024
+            );
+        } else {
+            log::info!(
+                "Chip: {} (Code Flash: {}KiB)",
+                self.chip,
+                self.chip.flash_size / 1024,
+            );
+        }
         log::info!(
             "Chip UID: {}",
             self.chip_uid
@@ -81,17 +101,18 @@ impl<T: Transport> Flashing<T> {
                 .join("-")
         );
         log::info!(
-            "BTVER(bootloader ver): {}",
-            self.bootloader_version[1..]
-                .iter()
-                .copied()
-                .map(|x| format!("{}", x))
-                .collect::<Vec<_>>()
-                .join(".")
+            "BTVER(bootloader ver): {:x}{:x}.{:x}{:x}",
+            self.bootloader_version[0],
+            self.bootloader_version[1],
+            self.bootloader_version[2],
+            self.bootloader_version[3]
         );
 
-        log::info!("Code Flash protected: {}", self.code_flash_protected);
+        if self.chip.support_code_flash_protect() {
+            log::info!("Code Flash protected: {}", self.code_flash_protected);
+        }
         self.dump_config()?;
+
         Ok(())
     }
 
@@ -181,6 +202,52 @@ impl<T: Transport> Flashing<T> {
         Ok(())
     }
 
+    pub fn reset_config(&mut self) -> Result<()> {
+        let read_conf = Command::read_config(CFG_MASK_RDPR_USER_DATA_WPR);
+        let resp = self.transport.transfer(read_conf)?;
+        anyhow::ensure!(resp.is_ok(), "read_config failed");
+
+        let mut raw = resp.payload()[2..].to_vec();
+
+        log::info!("Current config registers: {}", hex::encode(&raw));
+
+        for reg_desc in &self.chip.config_registers {
+            if let Some(reset) = reg_desc.reset {
+                raw.pwrite_with(reset, reg_desc.offset, scroll::LE)?;
+            }
+        }
+
+        log::info!("Reset config registers:   {}", hex::encode(&raw));
+        let write_conf = Command::write_config(CFG_MASK_RDPR_USER_DATA_WPR, raw);
+        let resp = self.transport.transfer(write_conf)?;
+        anyhow::ensure!(resp.is_ok(), "write_config failed");
+
+        // read back
+        let read_conf = Command::read_config(CFG_MASK_RDPR_USER_DATA_WPR);
+        let resp = self.transport.transfer(read_conf)?;
+        anyhow::ensure!(resp.is_ok(), "read_config failed");
+
+        Ok(())
+    }
+
+    pub fn dump_eeprom(&mut self) -> Result<Vec<u8>> {
+        const CHUNK: usize = 0x3a;
+        if self.chip.eeprom_size == 0 {
+            anyhow::bail!("Chip does not support EEPROM");
+        }
+        let mut ret: Vec<u8> = Vec::with_capacity(self.chip.eeprom_size as _);
+        let mut address = 0x0;
+        while address < self.chip.eeprom_size as u32 {
+            let cmd = Command::data_read(address, CHUNK as u16);
+            let resp = self.transport.transfer(cmd)?;
+            anyhow::ensure!(resp.is_ok(), "data_read failed");
+            address += CHUNK as u32;
+            assert!(resp.payload()[2..].len() == CHUNK);
+            ret.extend_from_slice(&resp.payload()[2..]);
+        }
+        Ok(ret)
+    }
+
     fn flash_chunk(&mut self, address: u32, raw: &[u8], key: [u8; 8]) -> Result<()> {
         let xored = raw.iter().enumerate().map(|(i, x)| x ^ key[i % 8]);
         let padding = rand::random();
@@ -218,8 +285,8 @@ impl<T: Transport> Flashing<T> {
     }
 
     pub fn erase_data(&mut self, _sectors: u16) -> Result<()> {
-        if self.chip.max_data_flash_size == 0 {
-            anyhow::bail!("chip doesn't support data flash");
+        if self.chip.eeprom_size == 0 {
+            anyhow::bail!("chip doesn't support data EEPROM");
         }
         unimplemented!("TODO")
     }
@@ -229,27 +296,39 @@ impl<T: Transport> Flashing<T> {
         let resp = self.transport.transfer(read_conf)?;
         anyhow::ensure!(resp.is_ok(), "read_config failed");
 
-        let regs = &resp.payload()[2..];
-        log::info!(
-            "RDPR:  0x{:02X}{:02X} ({})",
-            regs[0],
-            regs[1],
-            if regs[0] == 0xA5 {
-                "unprotected"
-            } else {
-                "protected"
+        let raw = &resp.payload()[2..];
+
+        for reg_def in &self.chip.config_registers {
+            let n = raw.pread_with::<u32>(reg_def.offset, LE)?;
+            println!("{}: 0x{:08X}", reg_def.name, n);
+
+            for (val, expain) in &reg_def.explaination {
+                if val == "_" || Some(n) == parse_number(val) {
+                    println!("  `- {}", expain);
+                    break;
+                }
             }
-        );
-        log::info!("USER:  0x{:02X}{:02X}", regs[2], regs[3]);
-        log::info!("USER0: 0x{:02X}{:02X}", regs[4], regs[5]);
-        log::info!("USER1: 0x{:02X}{:02X}", regs[6], regs[7]);
-        log::info!(
-            "WPR:   0x{:02X}{:02X}{:02X}{:02X}",
-            regs[8],
-            regs[9],
-            regs[10],
-            regs[11]
-        );
+
+            for field_def in &reg_def.fields {
+                let bit_width =
+                    (field_def.bit_range.start() - field_def.bit_range.end()) as u32 + 1;
+                let b = (n >> field_def.bit_range.end()) & (2_u32.pow(bit_width) - 1);
+                println!(
+                    "  [{}:{}] {} 0b{:b} (0x{:X})",
+                    field_def.bit_range.start(),
+                    field_def.bit_range.end(),
+                    field_def.name,
+                    b,
+                    b
+                );
+                for (val, expain) in &field_def.explaination {
+                    if val == "_" || Some(b) == parse_number(val) {
+                        println!("    `- {}", expain);
+                        break;
+                    }
+                }
+            }
+        }
 
         Ok(())
     }
